﻿;*******************************************************************************
; AutoHotkey Version:   1.1.03.00
; Автор:                Rakhmatullin A.I.
; Имя скрипта:          Rh_Switcher.ahk
; Версия:               1.0.4.0
; Дата:                 2018-10-01
;*******************************************************************************
; Автоматизация переключения языка аналогично Punto Switcher.
;-------------------------------------------------------------------------------

#Persistent
#SingleInstance force

#InstallKeybdHook

#HotkeyInterval 2000 ; default (msec).
#MaxHotkeysPerInterval 200 ; default = 70

SetTitleMatchMode RegEx

SetBatchLines, -1
SetKeyDelay, 0

;-------------------------------------------------------------------------------
; Глобальные данные для Translit!

gCharacterCount := 0    ; Счётчик нажатых символов

gCyrChars := "№АВГДЕЖЗИЙКЛМНОПРСТУФХЦЧШЩЪЫЬЯавгдежзийклмнопрстуфхцчшщъыьяБЮбюэЭ"";:?Ёё."
gLatChars := "#FDULT:PBQRKVYJGHCNEA{WXIO}SMZfdult;pbqrkvyjghcnea[wxio]smz<>,.'""@$^&~``/"

;-------------------------------------------------------------------------------
gLogName := "D:\Systems\AutoHK\Scripts\Rh_Switcher.log"

;gLog := FileOpen(gLogName, "a")
;gLog := FileOpen(gLogName, "w")
;MsgBox % gLog, A_LastError

gCurDT := ""
FormatTime, gCurDT, %A_Now%, "yyyy-MM-dd HH:mm:ss"
;MsgBox % gCurDT
;LogAdd(gCurDT)

;-------------------------------------------------------------------------------
LogAdd(aText)
{
  global gLogName

  ;MsgBox % aText
  ;FileAppend, aText, gLogName,
  FileAppend, %aText%, %gLogName%
  FileAppend, `n, %gLogName%

  return
} ; LogAdd

;-------------------------------------------------------------------------------
; Работа со счётчиком.

; Сброс счётчика
ResetCharacterCount()
{
  ;Critical
  global gCharacterCount
  ;MsgBox % Key = "%A_ThisHotKey%" Count = "%gCharacterCount%"
  gCharacterCount := 0

  return
} ; ResetCharacterCount

; Увеличение счётчика
IncCharacterCount(aKey)
{
  ;Critical
  global gCharacterCount

  ; TODO: Не увеличивать счётчик при начальных пробелах!

  gCharacterCount := gCharacterCount + 1
  if (gCharacterCount >= 100)
    gCharacterCount := 50

  ;LogAdd(aKey)

  return
} ; IncCharacterCount

; Уменьшение счётчика
DecCharacterCount()
{
  ;Critical
  global gCharacterCount

  gCharacterCount := gCharacterCount - 1
  if (gCharacterCount < 0)
    gCharacterCount := 0

  return
} ; DecCharacterCount

; Переключение языка Rh_Switcher.ahk
; на русский для правильной работы с буфером обмена.
{
  WinShow, ahk_id %A_ScriptHwnd%,
  SendMessage, 0x50,, 0x4190419,, ahk_id %A_ScriptHwnd%, ; Don't work with hide windows
  ;Sleep, 10
  WinHide, ahk_id %A_ScriptHwnd%,
}

;-------------------------------------------------------------------------------
; RAlt+LWin+R - Перезагрузка скрипта.
>!<#sc013::
  Run, %A_AhkPath% /restart %A_ScriptFullPath%, %A_WorkingDir%
  ; If successful, the reload will close this instance during the Sleep,
  Sleep 1000 ; so the line below will never be reached.
  ;MsgBox, The script could not be reloaded!
return

;-------------------------------------------------------------------------------
;LangRus = 0x00000419 ; Русский (RF)
;LangEnu = 0x00000409 ; Английский (US)
;LocaleRus = 0x4190419  ; Русский (RF)
;LocaleEnu = 0x4090409  ; Английский (US)

;-------------------------------------------------------------------------------
; Определение языка ввода в любом активном окне, включая консольное.
; © teadrinker, creature.ws
GetInputLangID(aWindow)
{
  if !(hWnd := WinExist(aWindow))
      return

  WinGetClass, Class

  if (Class == "ConsoleWindowClass")
  {
    WinGet, mConsolePID, PID
    DllCall("AttachConsole", Ptr, mConsolePID)
    VarSetCapacity(mBuffer, 16)
    DllCall("GetConsoleKeyboardLayoutName", Str, mBuffer)
    DllCall("FreeConsole")
    mLangID := "0x" . SubStr(mBuffer, -3)
  }
  else
    mLangID := DllCall("GetKeyboardLayout", Ptr, DllCall("GetWindowThreadProcessId", Ptr, hWnd, UInt, 0, Ptr), Ptr) & 0xFFFF

  return mLangID
} ; GetInputLangID

GetInputLangName(aLangID)
{
  Size := (DllCall("GetLocaleInfo", UInt, langId, UInt, 0x1001, UInt, 0, UInt, 0) * 2)   ; LOCALE_SENGLANGUAGE := 0x1001
  VarSetCapacity(mLangName, Size, 0)
  DllCall("GetLocaleInfo", UInt, aLangID, UInt, 0x1001, Str, mLangName, UInt, Size)

  return mLangName
} ; GetInputLangName

SwitchLocale(aLangID)
{
  if (aLangID = 0x0409) ; Enu
    PostMessage, 0x50,, 0x4190419,, A
  else if (aLangID = 0x0419) ; Rus
    PostMessage, 0x50,, 0x4090409,, A
  ;MsgBox % aLangID
  ;MsgBox % GetInputLangName(aLangID)

  Sleep, 100
} ; SwitchLocale

;-------------------------------------------------------------------------------
KeySwitchLocale(aKeyDownUp = 0)
{
  Critical
  ResetCharacterCount()

  static vTickLast
  if (aKeyDownUp = 1)
  {
    vTickLast := A_TickCount
    return
  }
  if (aKeyDownUp = 2)
  {
    if (A_TickCount - vTickLast > 200)
      return
  }

  mLangID := GetInputLangID("A")
  ;MsgBox % mLangID

  SwitchLocale(mLangID)
  ;Sleep 2000
  ;MsgBox % GetInputLangName(mLangID)
} ; KeySwitchLocale

;-------------------------------------------------------------------------------
; Транслитерация текста
TranslitText(aText)
{
  mCyrChars := "№АВГДЕЖЗИЙКЛМНОПРСТУФХЦЧШЩЪЫЬЯавгдежзийклмнопрстуфхцчшщъыьяБЮбюЭэ?Ёё"
  mLatChars := "#FDULT:PBQRKVYJGHCNEA{WXIO}SMZfdult;pbqrkvyjghcnea[wxio]smz<>,.""'&~``"

  Result := ""

  Loop, parse, aText
  {
    mPos := InStr(mLatChars, A_LoopField, 1)
    if (mPos > 0)
      Result .= SubStr(mCyrChars, mPos, 1)
    else {
      mPos := InStr(mCyrChars, A_LoopField, 1)
      if (mPos > 0)
        Result .= SubStr(mLatChars, mPos, 1)
      else
        Result .= A_LoopField
    }
    ;MsgBox % Result
  }

  return, Result
} ; TranslitText

; Перекодировка текста
XlatText(aText, aFrom, aTo)
{
  if (aFrom = "" or aTo = "")
    return, aText

  ;MsgBox % aFrom

  Result := ""

  Loop, parse, aText
  {
    mPos := InStr(aFrom, A_LoopField, 1)
    if (mPos > 0)
      Result .= SubStr(aTo, mPos, 1)
    else
      Result .= A_LoopField
    ;MsgBox % Result
  }

  return, Result
} ; XlatText

CheckTextLang(aText, aFrom, aTo)
{
  ;mRusLat := "abcdefghijklmnopqrstuvwxyz[].,;'``"
  ;          ."ABCDEFGHIJKLMNOPQRSTUVWXYZ{}<>:""~"

  if (aFrom = "" or aTo = "")
    return, 0

  ; Учёт инициалов в неправильной раскладке
  mStr := Trim(aText)
  mLen := StrLen(mStr)
  if (mLen = 2 and SubStr(mStr, 2, 1) = "/")
    return, -1
  else if (mLen = 4 and SubStr(mStr, 2, 1) = "/" and SubStr(mStr, 4, 1) = "/")
    return, -1

  ; Подсчёт символов в каждой раскладке
  Loop, parse, aText
  {
    mFromPos += InStr(aFrom, A_LoopField, 1)
    mToPos += InStr(aTo, A_LoopField, 1)
  }

  if (mFromPos > mToPos)
    return, 1
  else
    return, -1
} ; CheckTextLang

DoTranslit(aSelected = 0, aLastWord = 0, aXlat = 0, aFrom = "", aTo = "")
{
  Critical

  global gCharacterCount
  CharCount := gCharacterCount
  ;ResetCharacterCount()

  TempClip := ClipboardAll
  Clipboard := ""

  if (aSelected = 0) {
    if (CharCount > 0)
      Send, +{Left %CharCount%}
    else {
      if (aLastWord > 0)
        Send, ^+{Left}
      else
        return
    }
    ;Send, ^{Left}
    ;Send, +{End}
  }
  Send, ^{Ins}
  Sleep, 10
  ClipWait, 0

  ;if (aSelected = 0) {
  ;  if (CharCount > 0) {
  ;    Send, {Right %CharCount%}
  ;    Send, {BS %CharCount%}
  ;  }
  ;}

  SelText := Clipboard
  if (SelText = "") then
    return

  ;MsgBox % SelText
  ;MsgBox % SubStr(SelText, 1, 1)
  ;SelText := ErrorLevel ? GetWord() : Clipboard

  if (aXlat = 0)
    Result := TranslitText(SelText)
  else if (aFrom != "" and aTo != "") {
    if (aXlat = 1)
      Result := XlatText(SelText, aFrom, aTo)
    else {
      mKind := CheckTextLang(SelText, aFrom, aTo)
      if (mKind > 0)
        Result := XlatText(SelText, aFrom, aTo)
      else if (mKind < 0)
        Result := XlatText(SelText, aTo, aFrom)
      else
        Result := SelText
    }
  }

  ;MsgBox % Result

  PostMessage, 0x50, 2,,, A

  ;SendInput, +{Del}
  ;Sleep, 10
  ;ClipWait, 0

  Clipboard := Result
  Sleep, 10
  ClipWait, 0
  ;SendInput, +{Ins}
  ;MsgBox % Result

  ;SendInput, %Result%
  ;Send, {Raw}%Result%
  SendInput, {Raw}%Result%
  Sleep, 100

  Clipboard := TempClip
  Sleep, 10
  ClipWait, 0

  return
} ; DoTranslit

;-------------------------------------------------------------------------------
; ОБРАБОТКА КЛАВИШ
;-------------------------------------------------------------------------------
; Текущий язык ввода.
;<+CapsLock:: MsgBox % gCharacterCount
;<+CapsLock:: MsgBox % GetInputLangName(GetInputLangID("A"))

; Переключение языка Enu ↔ Rus
<!CapsLock::
  Critical
  LangID := GetInputLangID("A")
  ;MsgBox % LangID

  SwitchLocale(LangID)
  ;MsgBox % GetInputLangName(LangID)
return

$~RControl:: KeySwitchLocale(1)
$~RControl up:: KeySwitchLocale(2)

;-------------------------------------------------------------------------------
; 0-9
~sc00B::
~sc002::
~sc003::
~sc004::
~sc005::
~sc006::
~sc007::
~sc008::
~sc009::
~sc00A::
~+sc00B::
~+sc002::
~+sc003::
~+sc004::
~+sc005::
~+sc006::
~+sc007::
~+sc008::
~+sc009::
~+sc00A::
  IncCharacterCount(A_ThisHotKey)
return

; A-Z
~sc01E::
~sc030::
~sc02E::
~sc020::
~sc012::
~sc021::
~sc022::
~sc023::
~sc017::
~sc024::
~sc025::
~sc026::
~sc032::
~sc031::
~sc018::
~sc019::
~sc010::
~sc013::
~sc01F::
~sc014::
~sc016::
~sc02F::
~sc011::
~sc02D::
~sc015::
~sc02C::
~+sc01E::
~+sc030::
~+sc02E::
~+sc020::
~+sc012::
~+sc021::
~+sc022::
~+sc023::
~+sc017::
~+sc024::
~+sc025::
~+sc026::
~+sc032::
~+sc031::
~+sc018::
~+sc019::
~+sc010::
~+sc013::
~+sc01F::
~+sc014::
~+sc016::
~+sc02F::
~+sc011::
~+sc02D::
~+sc015::
~+sc02C::
  IncCharacterCount(A_ThisHotKey)
return

; Symbols:
~sc027::
~sc00D::
~sc033::
~sc00C::
~sc034::
~sc035::
~sc029::
~sc01A::
~sc02B::
~sc01B::
~sc028::
~+sc027::
~+sc00D::
~+sc033::
~+sc00C::
~+sc034::
~+sc035::
~+sc029::
~+sc01A::
~+sc02B::
~+sc01B::
~+sc028::
  IncCharacterCount(A_ThisHotKey)
return

; Space
~Space::
~+Space::
~^+Space::
  IncCharacterCount(A_ThisHotKey)
return

; Modifiers
~LControl::
;~RControl::
~LAlt::
~RAlt::
;~LShift::
;~RShift::
~LWin::
~RWin::
  ResetCharacterCount()
return

; Backspace
~Backspace::
  DecCharacterCount()
  ;ResetCharacterCount()
return

; Specials
~PrintScreen::
;~Pause::
;~CapsLock::
~ScrollLock::
~NumLock::
~Tab::
~Enter::
~Escape::
~Delete::
~Insert::
~Home::
~End::
~PgUp::
~PgDn::
~Up::
~Down::
~Left::
~Right::
  ResetCharacterCount()
return

; Media
~Sleep::
  ResetCharacterCount()
return

; Mouse
~LButton::
~RButton::
~MButton::
~XButton1::
~XButton2::
~WheelUp::
~WheelDown::
~WheelLeft::
~WheelRight::
  ResetCharacterCount()
return

; Numpad

#If (GetKeyState("NumLock", "T") = 1)

~Numpad0::
~Numpad1::
~Numpad2::
~Numpad3::
~Numpad4::
~Numpad5::
~Numpad6::
~Numpad7::
~Numpad8::
~Numpad9::
~NumpadDot::
  ;MsgBox % A_ThisHotKey
  ;MsgBox % A_ThisLabel
  IncCharacterCount(A_ThisHotKey)
  ;ResetCharacterCount()
return

#If (GetKeyState("NumLock", "T") = 0)
~NumpadIns::
~NumpadEnd::
~NumpadDown::
~NumpadPgDn::
~NumpadLeft::
~NumpadClear::
~NumpadRight::
~NumpadHome::
~NumpadUp::
~NumpadPgUp::
~NumpadDel::
  ;MsgBox % A_ThisHotKey
  ResetCharacterCount()
return

~NumpadAdd::
~NumpadSub::
~NumpadMult::
~NumpadDiv::
  IncCharacterCount(A_ThisHotKey)
return

~NumpadEnter::
  ResetCharacterCount()
return

#If ; Команды для всех.

;-------------------------------------------------------------------------------
; Транслитерация текста.

;Pause::   DoTranslit(0, 0)  ; Translit last type
Pause::   DoTranslit(0, 0, 2, gCyrChars, gLatChars)  ; Cyr-from/to-Lat last type
<!Pause:: DoTranslit(1, 0)  ; Translit selection
>!Pause:: DoTranslit(0, 1)  ; Translit last word
<!<+Pause:: DoTranslit(0, 0)  ; Translit last type
<+Pause:: DoTranslit(0, 0, 1, gCyrChars, gLatChars)  ; Cyr-to-Lat last type
>+Pause:: DoTranslit(0, 0, 1, gLatChars, gCyrChars)  ; Lat-to-Cyr last type

;-------------------------------------------------------------------------------
